#############################################################################
# Verbosity Control in builds
# Information below taken from /vobs/mw/mw/make.noise
#############################################################################
MAKE_VERBOSE = TRUE

ifndef MAKE_VERBOSE
	NO_SHORT_ECHO = TRUE
endif
 
ifdef NO_SHORT_ECHO
 
ECHOOBJ    = @
ECHOTARGET = @
ECHOSOURCE = @
ECHOLIB    = @
ECHONOISE  = @true
 
else
 
  ifndef DIRNOISE
    DIRNOISE := $(shell pwd)
  endif
 
ECHOOBJ    = @echo "[ ${DIRNOISE} : $(notdir $<) --> $(notdir $@) ]"
ECHOTARGET = @echo "[ ${DIRNOISE} : $(notdir $@) ]"
ECHOSOURCE = @echo "[ ${DIRNOISE} : $(notdir $<) ]"
ECHOLIB    = @echo "[ -- ${DIRNOISE} : $(notdir $@) ]"
ECHONOISE  = @echo
 
endif
 
ifeq "$(strip ${MAKE_VERBOSE})"  "false"
.SILENT:
else
DASH_VERBOSE=-v
endif

#############################################################################
# Directories for Installation
#############################################################################

ifeq (${COMPILE_OPTION}, debug)
    OBJ = objd
    DFLAGS  = -g  ${MWOS}
    LDFLAGS = $(DFLAGS) -L${O}
else
    OBJ = obj
    DFLAGS  = ${MWOS}
    LDFLAGS = $(DFLAGS) -L${O} -Xlinker -s
endif

O=${OBJ}/i386
BINDIR  = ./bin

#############################################################################
# Flags for Installation
#############################################################################

CFILES		= 		\
              messages.cpp  	\
              pam_sso.cpp  	\
              cryptpc.cpp       \
              rotlr.cpp    	\
              sha1.cpp 		\

HFILES		= 		\
              messages.h  	\
              crypto.h  	\
              des.h    		\
              pswdsync.h    	\
              pam_sso.h    	\
              nttype.h    	\
              sha1.h 		\
	
OFILES		= 		\
              ${O}/messages.o   \
              ${O}/pam_sso.o    \
              ${O}/cryptpc.o  	\
              ${O}/rotlr.o    	\
              ${O}/sha1.o 	\

SSOSHAREDLIB     = ${O}/pam_sso.so.1
BINPLACEDAS      = ${BINDIR}/pam_sso.${EXTENSION}

SSOLIB     	 = ${SSOSHAREDLIB}

#############################################################################

CC		    = gcc 

#PLATFORM_CFLAGS = 

OPTFLAGS= -ansi

OPTIMIZATION = 

EXTRALIBS = -L$(GNUDIR) -lgcc -lpam -lsocket -lnsl -L${TRIPLEDESDIR} -l3des

ifeq (${COMPILE_OPTION}, debug)
	DFLAGS		= -g 
	BUILDMSG = "*** Building shared pam_sso module"
else
	DFLAGS		=	
	OPTIMIZATION = -O3
	BUILDMSG = "*** Building static pam_sso module"
endif

IFLAGS      	= -I /usr/include 

LDFLAGS		= $(DFLAGS) -L${O}  -G

DEFINES 	= -DRENTRANT ${DMWOS} -D_TIMESPEC_T_ 

CFLAGS		= $(OPTFLAGS) $(DFLAGS) ${PLATFORM_CFLAGS} $(IFLAGS) $(DEFINES)

#############################################################################
# .sfx.o rules 
#############################################################################
.SUFFIXES: .cpp
.cpp.o:
	$(CC) $(CFLAGS) -c $<x
#############################################################################

all: alldirs info ssolib

allinst: alldirs info ssolib install

alldirs: ${O} ${BINDIR}

info:
	@echo
	@echo ${BUILDMSG}
	@echo

ssolib: ${SSOLIB}
	$(CC) $(LDFLAGS) -o ${SSOLIB} $(OFILES) $(EXTRALIBS) 
	cp ${SSOLIB} ${BINPLACEDAS}

${SSOLIB}: $(OFILES)

install:
	chown root $(SSOLIB)
	cp $(SSOLIB) $(SECUREDIR)

${O}/rotlr.o : ../common/rotlr.cpp 
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $< 

${O}/sha1.o : ../common/sha1.cpp ../common/sha1.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/cryptpc.o : cryptpc.cpp ../common/crypto.h pam_sso.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/pam_sso.o : pam_sso.cpp pam_sso.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O}/messages.o : ../common/messages.cpp ../common/messages.h
	${ECHONOISE} "[ $< -> $@ ]"
	$(CC) $(CFLAGS) -c -o $@ $<

${O} ${BINDIR}:
	${ECHONOISE} "[ $@ ]"
	mkdir -p $@

clean:
	rm -f $(OFILES) $(SSOLIB) *~ core

realclean: clean
	rm -rf $(OFILES) $(SSOLIB) *~ core

